package ScalarTypes::NumericTypes;

# Load the Perl pragmas.
use 5.030000;
use strict;
use warnings;

# Set the package version. 
our $VERSION = '0.05';

# Load the Perl pragma Exporter.
use Exporter;

# Base class of this module.
our @ISA = qw(Exporter);

# Exporting the implemented subroutines.
our @EXPORT = qw(
                 is_unsigned_float
                 is_signed_float
                 is_unsigned_int
                 is_signed_int
                 is_binary
                 is_hex
                 is_upper_hex
                 is_lower_hex
                 is_roman
);

#------------------------------# 
# Subroutine is_unsigned_float #
#------------------------------# 
sub is_unsigned_float {
    # Assign subroutine argument to local variable.
    my $input = $_[0];
    # Set output variable.
    my $is_float = undef;
    # Set regex pattern using digits match.
    my $re = qr/^(([0-9]{1}|[1-9]{1}[0-9]+)[.][0-9]+)$/;
    # Check input with regex pattern.
    if ($input =~ $re) {
        $is_float = 1;
    } else {
        $is_float = 0;
    };
    # Return 0 (false) or 1 (true).
    return $is_float; 
};

#----------------------------# 
# Subroutine is_signed_float #
#----------------------------# 
sub is_signed_float {
    # Assign subroutine argument to local variable.
    my $input = $_[0];
    # Set output variable.
    my $is_float = undef;
    # Set regex pattern using digits match.
    my $re = qr/^([+-]([0-9]{1}|[1-9]{1}[0-9]+)[.][0-9]+)$/;
    # Check input with regex pattern.
    if ($input =~ $re) {
        $is_float = 1;
    } else {
        $is_float = 0;
    };
    # Return 0 (false) or 1 (true).
    return $is_float; 
};

#----------------------------# 
# Subroutine is_unsigned_int #
#----------------------------# 
sub is_unsigned_int {
    # Assign the subroutine argument to the local variable.
    my $input = $_[0];
    # Declare the return variable.
    my $is_int = undef;
    # Set the Perl conform regex pattern.
    my $re = qr/^(([1-9][0-9]*)|0)$/;
    # Check the input with the regex pattern.
    if ($input =~ $re) {
        # Set result to true.
        $is_int = 1;
    } else {
        # Set result to false.
        $is_int = 0;
    };
    # Return 0 (false) or 1 (true).
    return $is_int; 
};

#--------------------------# 
# Subroutine is_signed_int #
#--------------------------# 
sub is_signed_int {
    # Assign the subroutine argument to the local variable.
    my $input = $_[0];
    # Declare the return variable.
    my $is_int = undef;
    # Set the Perl conform regex pattern.
    my $re = qr/^([+-]([1-9][0-9]*|0))$/;
    # Check the input with the regex pattern.
    if ($input =~ $re) {
        # Set result to true.
        $is_int = 1;
    } else {
        # Set result to false.
        $is_int = 0;
    };
    # Return 0 (false) or 1 (true).
    return $is_int; 
};

#------------------------------------------------------------------------------# 
# Subroutine is_hex                                                            #
#                                                                              #
# Description:                                                                 #
# The subroutine checks whether the specified argument is a valid hexadecimal  #
# number. A hexadecimal number in the context of this method is a number       #
# consisting of integers from 0-9, lower case characters from a-f or upper     #
# case characters from A-F. Spaces before and after the number itself are not  #
# allowed. The subroutine returns 1 (true) or 0 (false) based on the check.    #
#------------------------------------------------------------------------------# 
sub is_hex {
    # Assign the subroutine argument to the local variable.
    my $arg = (defined $_[0] ? $_[0] : '');
    # Define the Perl conform regex pattern.
    my $re = qr/^([0-9a-fA-F]+)$/;
    # Check the argument with the regex pattern.
    my $is_hex = (($arg =~ $re) ? 1 : 0);
    # Return 0 (false) or 1 (true).
    return $is_hex;
};

#------------------------------------------------------------------------------# 
# Subroutine is_upper_hex                                                      #
#                                                                              #
# Description:                                                                 #
# The subroutine checks whether the specified argument is a valid hexadecimal  #
# number. A hexadecimal number in the context of this method is a number       #
# consisting of integers from 0-9 or upper case characters from A-F. Spaces    #
# before and after the number itself are not allowed. The subroutine returns   #
# 1 (true) or 0 (false) based on the check.                                    #
#------------------------------------------------------------------------------# 
sub is_upper_hex {
    # Assign the subroutine argument to the local variable.
    my $arg = (defined $_[0] ? $_[0] : '');
    # Define the Perl conform regex pattern.
    my $re = qr/^([0-9A-F]+)$/;
    # Check the argument with the regex pattern.
    my $is_uc_hex = (($arg =~ $re) ? 1 : 0);
    # Return 0 (false) or 1 (true).
    return $is_uc_hex;
};

#------------------------------------------------------------------------------# 
# Subroutine is_lower_hex                                                      #
#                                                                              #
# Description:                                                                 #
# The subroutine checks whether the specified argument is a valid hexadecimal  #
# number. A hexadecimal number in the context of this method is a number       #
# consisting of integers from 0-9 or lower case characters from a-f. Spaces    #
# before and after the number itself are not allowed. The subroutine returns   #
# 1 (true) or 0 (false) based on the check.                                    #
#------------------------------------------------------------------------------# 
sub is_lower_hex {
    # Assign the subroutine argument to the local variable.
    my $arg = (defined $_[0] ? $_[0] : '');
    # Define the Perl conform regex pattern.
    my $re = qr/^([0-9a-f]+)$/;
    # Check the argument with the regex pattern.
    my $is_lc_hex = (($arg =~ $re) ? 1 : 0);
    # Return 0 (false) or 1 (true).
    return $is_lc_hex;
};

#----------------------# 
# Subroutine is_binary #
#----------------------# 
sub is_binary {
    # Assign the subroutine argument to the local variable.
    my $arg = (defined $_[0] ? $_[0] : '');
    # Define the Perl conform regex pattern.
    my $re = qr/^([01]+)$/;
    # Check the argument with the regex pattern.
    my $is_binary = (($arg =~ $re) ? 1 : 0);
    # Return 0 (false) or 1 (true).
    return $is_binary;
};

#---------------------# 
# Subroutine is_roman #
#---------------------# 
sub is_roman {
    # Assign the subroutine argument to the local variable.
    my $arg = (defined $_[0] ? $_[0] : '');
    # Define the Perl conform regex pattern.
    my $re = qr/^([IVXLCDM]+)$/;
    # Check the argument with the regex pattern.
    my $is_binary = (($arg =~ $re) ? 1 : 0);
    # Return 0 (false) or 1 (true).
    return $is_binary;
};

1;

__END__
# Below is the package documentation.

=head1 NAME

ScalarTypes::NumericTypes - Perl extension for numeric types of Perl scalars

=head1 SYNOPSIS

  use ScalarTypes::NumericTypes;

  # Declare the teststring.
  my $teststring = undef;

  # Test unsigned integer.
  $teststring = "12345678"; # Valid unsigned integer.
  $testresult = is_unsigned_int($teststring); # Returns 1 (true)
  print $testresult . "\n";

  # Test signed integer.
  $teststring = "-12345678"; # Valid signed integer. 
  $testresult = is_signed_int($teststring); # Returns 1 (true)
  print $testresult . "\n";

  # Test unsigned float.
  $teststring = "1234.5678"; # Valid unsigned float. 
  $testresult = is_unsigned_float($teststring); # Returns 1 (true)
  print $testresult . "\n";

  # Test signed float.
  $teststring = "+1234.5678"; # Valid signed float.
  $testresult = is_signed_float($teststring); # Returns 1 (true)
  print $testresult . "\n";

  # and so on for the other methods

=head1 DESCRIPTION

=head2 Implemented Methods

=over 4 

=item * is_signed_integer()

=item * is_unsigned_integer()

=item * is_signed_float()

=item * is_unsigned_float()

=item * is_binary()

=item * is_hex()

=item * is_upper_hex()

=item * is_lower_hex()

=item * is_roman()

=back

All of the implemented methods return 1 (true) or return 0 (false). A
subroutine argument is necessary otherwise an exception could be thrown. 

=head2 is_unsigned_int()

Following scalars return 1 (true):

    '0'        -> 1
    '6430'     -> 1
    '12345678' -> 1

Following scalars return 0 (false):

    '01234567' -> 0
    ' 823467'  -> 0
    '521496 '  -> 0

A leading 0 and spaces before and after result in not valid result.

=head2 is_signed_int()

Following scalars return 1 (true):

    '+1'       -> 1
    '-6430'    -> 1
    '+1245678' -> 1

Following scalars return 0 (false):

    '-0'       -> 0
    ' +26367'  -> 0
    '-52496 '  -> 0

=head2 is_unsigned_float()

Following scalars return 1 (true):

    '0.0'      -> 1
    '0.9'      -> 1
    '1.645'    -> 1
    '124.567'  -> 1

Following scalars return 0 (false):

    '.0'       -> 0
    '0.'       -> 0
    ' 1.3'     -> 0
    '3.1 '     -> 0

=head2 is_signed_float()

Following scalars return 1 (true):

    '+0.9      -> 1
    '-1.645'   -> 1
    '+24.567'  -> 1

Following scalars return 0 (false):

    '+.0'      -> 0
    '-0.'      -> 0
    ' +2.3'    -> 0
    '-3.4 '    -> 0

=head2 is_hex()
 
The subroutine checks whether the specified argument is a valid hexadecimal 
number. A hexadecimal number in the context of this method is a number      
consisting of integers from 0-9, lower case characters from a-f or upper    
case characters from A-F. Spaces before and after the number itself are not 
allowed. The subroutine returns 1 (true) or 0 (false) based on the check.   

=head2 is_upper_hex()

Same as is_hex(). Only uper case hex characters are valid.

=head2 is_lower_hex()

Same as is_hex(). Only lower case hex characters are valid.

=head2 is_binary()

The method returns true on a valid binary. A valid binary consist of 0 and 1.

=head2 is_roman()

The method returns true on a valid roman number. A valid roman number consist 
of uper case letters I (=1), V (=5), X (=10), L (=50), C (=100), D (=500) and
M (=1000).

=head1 SEE ALSO

Perl documentation
Tutorials about Regular Expressions

=head1 AUTHOR

Dr. Peter Netz, E<lt>ztenretep@cpan.orgE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2022 by Dr. Peter Netz

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.30.0 or,
at your option, any later version of Perl 5 you may have available.

=cut
